/* $Id: xbuf_descriptor.h,v 1.2 2003/10/13 22:16:25 meinelte Exp $ */
/******************************************************************************
*
*                       ****************************
*                       ** Copyright Xilinx, Inc. **
*                       ** All rights reserved.   **
*                       ****************************
*
* FILENAME:
*
* xbuf_descriptor.h
*
* DESCRIPTION:
*
* This file contains the interface for the XBufDescriptor component.
* The XBufDescriptor component is a passive component that only maps over
* a buffer descriptor data structure shared by the scatter gather DMA hardware
* and software. The component's primary purpose is to provide encapsulation of
* the buffer descriptor processing.  See the source file xbuf_descriptor.c for
* details.
*
* NOTES:
*
* Most of the functions of this component are implemented as macros in order
* to optimize the processing.  The names are not all uppercase such that they
* can be switched between macros and functions easily.
*
******************************************************************************/

#ifndef XBUF_DESCRIPTOR_H    /* prevent circular inclusions */
#define XBUF_DESCRIPTOR_H    /* by using protection macros */

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "xdma_channel_i.h"

/************************** Constant Definitions *****************************/

/* The following constants allow access to all fields of a buffer descriptor
 * and are necessary at this level of visibility to allow macros to access
 * and modify the fields of a buffer descriptor.  It is not expected that the
 * user of a buffer descriptor would need to use these constants.
 */

#define XBD_DEVICE_STATUS_OFFSET    0
#define XBD_CONTROL_OFFSET          1
#define XBD_SOURCE_OFFSET           2
#define XBD_DESTINATION_OFFSET      3
#define XBD_LENGTH_OFFSET           4
#define XBD_STATUS_OFFSET           5
#define XBD_NEXT_PTR_OFFSET         6
#define XBD_ID_OFFSET               7
#define XBD_FLAGS_OFFSET            8
#define XBD_RQSTED_LENGTH_OFFSET    9

#define XBD_SIZE_IN_WORDS           10

/*
 * The following constants define the bits of the flags field of a buffer
 * descriptor
 */

#define XBD_FLAGS_LOCKED_MASK       1UL

/**************************** Type Definitions *******************************/

typedef Xuint32 XBufDescriptor[XBD_SIZE_IN_WORDS];

/***************** Macros (Inline Functions) Definitions *********************/

/* each of the following macros are named the same as functions rather than all
 * upper case in order to allow either the macros or the functions to be
 * used, see the source file xbuf_descriptor.c for documentation
 */

#define XBufDescriptor_Initialize(InstancePtr)                  \
{                                                               \
    (*((Xuint32 *)InstancePtr + XBD_CONTROL_OFFSET) = 0);       \
    (*((Xuint32 *)InstancePtr + XBD_SOURCE_OFFSET) = 0);        \
    (*((Xuint32 *)InstancePtr + XBD_DESTINATION_OFFSET) = 0);   \
    (*((Xuint32 *)InstancePtr + XBD_LENGTH_OFFSET) = 0);        \
    (*((Xuint32 *)InstancePtr + XBD_STATUS_OFFSET) = 0);        \
    (*((Xuint32 *)InstancePtr + XBD_DEVICE_STATUS_OFFSET) = 0); \
    (*((Xuint32 *)InstancePtr + XBD_NEXT_PTR_OFFSET) = 0);      \
    (*((Xuint32 *)InstancePtr + XBD_ID_OFFSET) = 0);            \
    (*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET) = 0);         \
    (*((Xuint32 *)InstancePtr + XBD_RQSTED_LENGTH_OFFSET) = 0); \
}

#define XBufDescriptor_GetControl(InstancePtr)   \
    (Xuint32)(*((Xuint32 *)InstancePtr + XBD_CONTROL_OFFSET))

#define XBufDescriptor_SetControl(InstancePtr, Control)  \
    (*((Xuint32 *)InstancePtr + XBD_CONTROL_OFFSET) = (Xuint32)Control)

#define XBufDescriptor_IsLastControl(InstancePtr) \
    (Xboolean)((*((Xuint32 *)InstancePtr + XBD_CONTROL_OFFSET) & \
               XDC_CONTROL_LAST_BD_MASK) == XDC_CONTROL_LAST_BD_MASK)

#define XBufDescriptor_SetLast(InstancePtr) \
    (*((Xuint32 *)InstancePtr + XBD_CONTROL_OFFSET) |= XDC_CONTROL_LAST_BD_MASK)

#define XBufDescriptor_GetSrcAddress(InstancePtr) \
    ((Xuint32 *)(*((Xuint32 *)InstancePtr + XBD_SOURCE_OFFSET)))

#define XBufDescriptor_SetSrcAddress(InstancePtr, Source) \
    (*((Xuint32 *)InstancePtr + XBD_SOURCE_OFFSET) = (Xuint32)Source)

#define XBufDescriptor_GetDestAddress(InstancePtr) \
    ((Xuint32 *)(*((Xuint32 *)InstancePtr + XBD_DESTINATION_OFFSET)))

#define XBufDescriptor_SetDestAddress(InstancePtr, Destination) \
    (*((Xuint32 *)InstancePtr + XBD_DESTINATION_OFFSET) = (Xuint32)Destination)

#define XBufDescriptor_GetLength(InstancePtr)                           \
    (Xuint32)(*((Xuint32 *)InstancePtr + XBD_RQSTED_LENGTH_OFFSET) -    \
              *((Xuint32 *)InstancePtr + XBD_LENGTH_OFFSET))

#define XBufDescriptor_SetLength(InstancePtr, Length)                       \
{                                                                           \
    (*((Xuint32 *)InstancePtr + XBD_LENGTH_OFFSET) = (Xuint32)(Length));    \
    (*((Xuint32 *)InstancePtr + XBD_RQSTED_LENGTH_OFFSET) = (Xuint32)(Length));\
}

#define XBufDescriptor_GetStatus(InstancePtr)    \
    (Xuint32)(*((Xuint32 *)InstancePtr + XBD_STATUS_OFFSET))

#define XBufDescriptor_SetStatus(InstancePtr, Status)    \
    (*((Xuint32 *)InstancePtr + XBD_STATUS_OFFSET) = (Xuint32)Status)

#define XBufDescriptor_IsLastStatus(InstancePtr) \
    (Xboolean)((*((Xuint32 *)InstancePtr + XBD_STATUS_OFFSET) & \
               XDC_STATUS_LAST_BD_MASK) == XDC_STATUS_LAST_BD_MASK)

#define XBufDescriptor_GetDeviceStatus(InstancePtr) \
    ((Xuint32)(*((Xuint32 *)InstancePtr + XBD_DEVICE_STATUS_OFFSET)))

#define XBufDescriptor_SetDeviceStatus(InstancePtr, Status) \
    (*((Xuint32 *)InstancePtr + XBD_DEVICE_STATUS_OFFSET) = (Xuint32)Status)

#define XBufDescriptor_GetNextPtr(InstancePtr) \
    (XBufDescriptor *)(*((Xuint32 *)InstancePtr + XBD_NEXT_PTR_OFFSET))

#define XBufDescriptor_SetNextPtr(InstancePtr, NextPtr) \
    (*((Xuint32 *)InstancePtr + XBD_NEXT_PTR_OFFSET) = (Xuint32)NextPtr)

#define XBufDescriptor_GetId(InstancePtr) \
    (Xuint32)(*((Xuint32 *)InstancePtr + XBD_ID_OFFSET))

#define XBufDescriptor_SetId(InstancePtr, Id) \
    (*((Xuint32 *)InstancePtr + XBD_ID_OFFSET) = (Xuint32)Id)

#define XBufDescriptor_GetFlags(InstancePtr) \
    (Xuint32)(*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET))

#define XBufDescriptor_SetFlags(InstancePtr, Flags) \
    (*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET) = (Xuint32)Flags)

#define XBufDescriptor_Lock(InstancePtr) \
    (*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET) |= XBD_FLAGS_LOCKED_MASK)

#define XBufDescriptor_Unlock(InstancePtr) \
    (*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET) &= ~XBD_FLAGS_LOCKED_MASK)

#define XBufDescriptor_IsLocked(InstancePtr) \
    (Xboolean) ((*((Xuint32 *)InstancePtr + XBD_FLAGS_OFFSET) & \
        XBD_FLAGS_LOCKED_MASK) == XBD_FLAGS_LOCKED_MASK)

/************************** Function Prototypes ******************************/

/* The following prototypes are provided to allow each of the functions to
 * be implemented as a function rather than a macro, and to provide the
 * syntax to allow users to understand how to call the macros, they are
 * commented out to prevent linker errors
 *

Xuint32 XBufDescriptor_Initialize(XBufDescriptor* InstancePtr);

Xuint32 XBufDescriptor_GetControl(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetControl(XBufDescriptor* InstancePtr, Xuint32 Control);

Xboolean XBufDescriptor_IsLastControl(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetLast(XBufDescriptor* InstancePtr);

Xuint32 XBufDescriptor_GetLength(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetLength(XBufDescriptor* InstancePtr, Xuint32 Length);

Xuint32 XBufDescriptor_GetStatus(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetStatus(XBufDescriptor* InstancePtr, Xuint32 Status);
Xboolean XBufDescriptor_IsLastStatus(XBufDescriptor* InstancePtr);

Xuint32 XBufDescriptor_GetDeviceStatus(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetDeviceStatus(XBufDescriptor* InstancePtr,
                                    Xuint32 Status);

Xuint32 XBufDescriptor_GetSrcAddress(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetSrcAddress(XBufDescriptor* InstancePtr,
                                  Xuint32 SourceAddress);

Xuint32 XBufDescriptor_GetDestAddress(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetDestAddress(XBufDescriptor* InstancePtr,
                                   Xuint32 DestinationAddress);

XBufDescriptor* XBufDescriptor_GetNextPtr(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetNextPtr(XBufDescriptor* InstancePtr,
                               XBufDescriptor* NextPtr);

Xuint32 XBufDescriptor_GetId(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetId(XBufDescriptor* InstancePtr, Xuint32 Id);

Xuint32 XBufDescriptor_GetFlags(XBufDescriptor* InstancePtr);
void XBufDescriptor_SetFlags(XBufDescriptor* InstancePtr, Xuint32 Flags);

void XBufDescriptor_Lock(XBufDescriptor* InstancePtr);
void XBufDescriptor_Unlock(XBufDescriptor* InstancePtr);
Xboolean XBufDescriptor_IsLocked(XBufDescriptor* InstancePtr);

void XBufDescriptor_Copy(XBufDescriptor* InstancePtr,
                         XBufDescriptor* DestinationPtr);

*/

#endif              /* end of protection macro */
